@@ -1,27 +1,31 @@
 //! Read configurations files.
 
-#![deny(missing_docs_in_private_items)]
+#![deny(clippy::missing_docs_in_private_items)]
 
-use std::{env, fmt, fs, io, path};
+use lazy_static::lazy_static;
+use std::default::Default;
 use std::io::Read;
-use syntax::{ast, codemap};
-use toml;
 use std::sync::Mutex;
+use std::{env, fmt, fs, io, path};
+use syntax::{ast, source_map};
+use toml;
 
 /// Get the configuration file from arguments.
 pub fn file_from_args(
-    args: &[codemap::Spanned<ast::NestedMetaItemKind>],
-) -> Result<Option<path::PathBuf>, (&'static str, codemap::Span)> {
-    for arg in args.iter().filter_map(|a| a.meta_item()) {
+    args: &[source_map::Spanned<ast::NestedMetaItemKind>],
+) -> Result<Option<path::PathBuf>, (&'static str, source_map::Span)> {
+    for arg in args.iter().filter_map(syntax::source_map::Spanned::meta_item) {
         if arg.name() == "conf_file" {
             return match arg.node {
                 ast::MetaItemKind::Word | ast::MetaItemKind::List(_) => {
                     Err(("`conf_file` must be a named value", arg.span))
                 },
-                ast::MetaItemKind::NameValue(ref value) => if let ast::LitKind::Str(ref file, _) = value.node {
-                    Ok(Some(file.to_string().into()))
-                } else {
-                    Err(("`conf_file` value must be a string", value.span))
+                ast::MetaItemKind::NameValue(ref value) => {
+                    if let ast::LitKind::Str(ref file, _) = value.node {
+                        Ok(Some(file.to_string().into()))
+                    } else {
+                        Err(("`conf_file` value must be a string", value.span))
+                    }
                 },
             };
         }
@@ -37,28 +41,13 @@ pub enum Error {
     Io(io::Error),
     /// Not valid toml or doesn't fit the expected conf format
     Toml(String),
-    /// Type error.
-    Type(
-        /// The name of the key.
-        &'static str,
-        /// The expected type.
-        &'static str,
-        /// The type we got instead.
-        &'static str,
-    ),
-    /// There is an unknown key is the file.
-    UnknownKey(String),
 }
 
 impl fmt::Display for Error {
-    fn fmt(&self, f: &mut fmt::Formatter) -> Result<(), fmt::Error> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> Result<(), fmt::Error> {
         match *self {
             Error::Io(ref err) => err.fmt(f),
             Error::Toml(ref err) => err.fmt(f),
-            Error::Type(key, expected, got) => {
-                write!(f, "`{}` is expected to be a `{}` but is a `{}`", key, expected, got)
-            },
-            Error::UnknownKey(ref key) => write!(f, "unknown key `{}`", key),
         }
     }
 }
@@ -77,10 +66,10 @@ macro_rules! define_Conf {
     ($(#[$doc: meta] ($rust_name: ident, $rust_name_str: expr, $default: expr => $($ty: tt)+),)+) => {
         pub use self::helpers::Conf;
         mod helpers {
+            use serde_derive::Deserialize;
             /// Type used to store lint configuration.
             #[derive(Deserialize)]
-            #[serde(rename_all="kebab-case")]
-            #[serde(deny_unknown_fields)]
+            #[serde(rename_all="kebab-case", deny_unknown_fields)]
             pub struct Conf {
                 $(#[$doc] #[serde(default=$rust_name_str)] #[serde(with=$rust_name_str)]
                           pub $rust_name: define_Conf!(TY $($ty)+),)+
@@ -92,12 +81,12 @@ macro_rules! define_Conf {
                 mod $rust_name {
                     use serde;
                     use serde::Deserialize;
-                    pub fn deserialize<'de, D: serde::Deserializer<'de>>(deserializer: D)
+                    crate fn deserialize<'de, D: serde::Deserializer<'de>>(deserializer: D)
                     -> Result<define_Conf!(TY $($ty)+), D::Error> {
                         type T = define_Conf!(TY $($ty)+);
                         Ok(T::deserialize(deserializer).unwrap_or_else(|e| {
-                            ::utils::conf::ERRORS.lock().expect("no threading here")
-                                                        .push(::utils::conf::Error::Toml(e.to_string()));
+                            crate::utils::conf::ERRORS.lock().expect("no threading here")
+                                                        .push(crate::utils::conf::Error::Toml(e.to_string()));
                             super::$rust_name()
                         }))
                     }
@@ -113,29 +102,6 @@ macro_rules! define_Conf {
     // hack to convert tts
     (TY $ty: ty) => { $ty };
 
-    // how to read the value?
-    (CONV i64, $value: expr) => { $value.as_integer() };
-    (CONV u64, $value: expr) => {
-        $value.as_integer()
-        .iter()
-        .filter_map(|&i| if i >= 0 { Some(i as u64) } else { None })
-        .next()
-    };
-    (CONV String, $value: expr) => { $value.as_str().map(Into::into) };
-    (CONV Vec<String>, $value: expr) => {{
-        let slice = $value.as_array();
-
-        if let Some(slice) = slice {
-            if slice.iter().any(|v| v.as_str().is_none()) {
-                None
-            } else {
-                Some(slice.iter().map(|v| v.as_str().expect("already checked").to_owned()).collect())
-            }
-        } else {
-            None
-        }
-    }};
-
     // provide a nicer syntax to declare the default value of `Vec<String>` variables
     (DEFAULT Vec<String>, $e: expr) => { $e.iter().map(|&e| e.to_owned()).collect() };
     (DEFAULT $ty: ty, $e: expr) => { $e };
@@ -152,16 +118,17 @@ define_Conf! {
         "DirectX",
         "ECMAScript",
         "GPLv2", "GPLv3",
-        "GitHub",
+        "GitHub", "GitLab",
         "IPv4", "IPv6",
         "JavaScript",
-        "NaN",
+        "NaN", "NaNs",
         "OAuth",
-        "OpenGL", "OpenSSH", "OpenSSL",
+        "OpenGL", "OpenSSH", "OpenSSL", "OpenStreetMap",
         "TrueType",
         "iOS", "macOS",
         "TeX", "LaTeX", "BibTeX", "BibLaTeX",
         "MinGW",
+        "CamelCase",
     ] => Vec<String>),
     /// Lint: TOO_MANY_ARGUMENTS. The maximum number of argument a function or method can have
     (too_many_arguments_threshold, "too_many_arguments_threshold", 7 => u64),
@@ -173,10 +140,22 @@ define_Conf! {
     (too_large_for_stack, "too_large_for_stack", 200 => u64),
     /// Lint: ENUM_VARIANT_NAMES. The minimum number of enum variants for the lints about variant names to trigger
     (enum_variant_name_threshold, "enum_variant_name_threshold", 3 => u64),
-    /// Lint: LARGE_ENUM_VARIANT. The maximum size of a emum's variant to avoid box suggestion
+    /// Lint: LARGE_ENUM_VARIANT. The maximum size of a enum's variant to avoid box suggestion
     (enum_variant_size_threshold, "enum_variant_size_threshold", 200 => u64),
     /// Lint: VERBOSE_BIT_MASK. The maximum allowed size of a bit mask before suggesting to use 'trailing_zeros'
     (verbose_bit_mask_threshold, "verbose_bit_mask_threshold", 1 => u64),
+    /// Lint: DECIMAL_LITERAL_REPRESENTATION. The lower bound for linting decimal literals
+    (literal_representation_threshold, "literal_representation_threshold", 16384 => u64),
+    /// Lint: TRIVIALLY_COPY_PASS_BY_REF. The maximum size (in bytes) to consider a `Copy` type for passing by value instead of by reference.
+    (trivial_copy_size_limit, "trivial_copy_size_limit", None => Option<u64>),
+    /// Lint: TOO_MANY_LINES. The maximum number of lines a function or method can have
+    (too_many_lines_threshold, "too_many_lines_threshold", 100 => u64),
+}
+
+impl Default for Conf {
+    fn default() -> Self {
+        toml::from_str("").expect("we never error on empty config files")
+    }
 }
 
 /// Search for the configuration file.
@@ -184,8 +163,13 @@ pub fn lookup_conf_file() -> io::Result<Option<path::PathBuf>> {
     /// Possible filename to search for.
     const CONFIG_FILE_NAMES: [&str; 2] = [".clippy.toml", "clippy.toml"];
 
-    let mut current = try!(env::current_dir());
-
+    // Start looking for a config file in CLIPPY_CONF_DIR, or failing that, CARGO_MANIFEST_DIR.
+    // If neither of those exist, use ".".
+    let mut current = path::PathBuf::from(
+        env::var("CLIPPY_CONF_DIR")
+            .or_else(|_| env::var("CARGO_MANIFEST_DIR"))
+            .unwrap_or_else(|_| ".".to_string()),
+    );
     loop {
         for config_file_name in &CONFIG_FILE_NAMES {
             let config_file = current.join(config_file_name);
@@ -195,8 +179,10 @@ pub fn lookup_conf_file() -> io::Result<Option<path::PathBuf>> {
                 Ok(ref md) if md.is_file() => return Ok(Some(config_file)),
                 // Return the error if it's something other than `NotFound`; otherwise we didn't
                 // find the project file yet, and continue searching.
-                Err(e) => if e.kind() != io::ErrorKind::NotFound {
-                    return Err(e);
+                Err(e) => {
+                    if e.kind() != io::ErrorKind::NotFound {
+                        return Err(e);
+                    }
                 },
                 _ => (),
             }
@@ -213,7 +199,7 @@ pub fn lookup_conf_file() -> io::Result<Option<path::PathBuf>> {
 ///
 /// Used internally for convenience
 fn default(errors: Vec<Error>) -> (Conf, Vec<Error>) {
-    (toml::from_str("").expect("we never error on empty config files"), errors)
+    (Conf::default(), errors)
 }
 
 /// Read the `toml` configuration file.
@@ -239,25 +225,14 @@ pub fn read(path: Option<&path::Path>) -> (Conf, Vec<Error>) {
         Err(err) => return default(vec![err.into()]),
     };
 
-    assert!(
-        ERRORS
-            .lock()
-            .expect("no threading -> mutex always safe")
-            .is_empty()
-    );
+    assert!(ERRORS.lock().expect("no threading -> mutex always safe").is_empty());
     match toml::from_str(&file) {
         Ok(toml) => (
             toml,
-            ERRORS
-                .lock()
-                .expect("no threading -> mutex always safe")
-                .split_off(0),
+            ERRORS.lock().expect("no threading -> mutex always safe").split_off(0),
         ),
         Err(e) => {
-            let mut errors = ERRORS
-                .lock()
-                .expect("no threading -> mutex always safe")
-                .split_off(0);
+            let mut errors = ERRORS.lock().expect("no threading -> mutex always safe").split_off(0);
             errors.push(Error::Toml(e.to_string()));
             default(errors)
         },
