@@ -1,13 +1,3 @@
-// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-
 #![deny(clippy::mem_discriminant_non_enum)]
 
 use std::mem;
@@ -35,7 +25,9 @@ fn main() {
     mem::discriminant(&rro);
 
     macro_rules! mem_discriminant_but_in_a_macro {
-        ($param:expr) => (mem::discriminant($param))
+        ($param:expr) => {
+            mem::discriminant($param)
+        };
     }
 
     mem_discriminant_but_in_a_macro!(&rro);
