@@ -1,26 +1,15 @@
-// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-
-
-
 #![warn(clippy::single_match)]
 
-fn dummy() {
-}
+fn dummy() {}
 
-fn single_match(){
+fn single_match() {
     let x = Some(1u8);
 
     match x {
-        Some(y) => { println!("{:?}", y); }
-        _ => ()
+        Some(y) => {
+            println!("{:?}", y);
+        },
+        _ => (),
     };
 
     let x = Some(1u8);
@@ -29,19 +18,19 @@ fn single_match(){
         // We suggest `if let Some(y) = x { .. }` because the macro
         // is expanded before we can do anything.
         Some(y) => println!("{:?}", y),
-        _ => ()
+        _ => (),
     }
 
-    let z = (1u8,1u8);
+    let z = (1u8, 1u8);
     match z {
         (2...3, 7...9) => dummy(),
-        _ => {}
+        _ => {},
     };
 
     // Not linted (pattern guards used)
     match x {
         Some(y) if y == 0 => println!("{:?}", y),
-        _ => ()
+        _ => (),
     }
 
     // Not linted (no block with statements in the single arm)
@@ -51,22 +40,25 @@ fn single_match(){
     }
 }
 
-enum Foo { Bar, Baz(u8) }
-use Foo::*;
+enum Foo {
+    Bar,
+    Baz(u8),
+}
 use std::borrow::Cow;
+use Foo::*;
 
 fn single_match_know_enum() {
     let x = Some(1u8);
-    let y : Result<_, i8> = Ok(1i8);
+    let y: Result<_, i8> = Ok(1i8);
 
     match x {
         Some(y) => dummy(),
-        None => ()
+        None => (),
     };
 
     match y {
         Ok(y) => dummy(),
-        Err(..) => ()
+        Err(..) => (),
     };
 
     let c = Cow::Borrowed("");
@@ -89,4 +81,4 @@ fn single_match_know_enum() {
     }
 }
 
-fn main() { }
+fn main() {}
