@@ -1,14 +1,3 @@
-// Copyright 2017 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-// run-rustfix
 // compile-pass
 
 // The output for humans should just highlight the whole span without showing
@@ -17,12 +6,10 @@
 // stripping away any starting or ending parenthesis characters—hence this
 // test of the JSON error format.
 
-
 #![deny(clippy::unused_unit)]
 #![allow(clippy::needless_return)]
 
 struct Unitter;
-
 impl Unitter {
     // try to disorient the lint with multiple unit returns and newlines
     pub fn get_unit<F: Fn() -> (), G>(&self, f: F, _g: G) ->
@@ -34,6 +21,7 @@ impl Unitter {
 }
 
 impl Into<()> for Unitter {
+    #[rustfmt::skip]
     fn into(self) -> () {
         ()
     }
