@@ -13,5 +13,5 @@
 fn main() {
     let functions: [Box<Fn() -> Option<()>>; 1] = [Box::new(|| None)];
 
-    let _: Option<Vec<()>> = functions.iter().map(|f| (*f)()).collect();
+    let _val: Option<Vec<()>> = functions.iter().map(|f| (*f)()).collect();
 }
