@@ -400,14 +400,6 @@ fn foo() {
     foo!(1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,);
 }
 
-// #2652
-// Preserve trailing comma inside macro, even if it looks an array.
-macro_rules! bar {
-    ($m:ident) => {
-        $m!([a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p, q, r, s, t, u, v, w, x, y, z]);
-    };
-}
-
 // #2830
 // Preserve trailing comma-less/ness inside nested macro.
 named!(
@@ -466,3 +458,16 @@ static ROOTED_TRACEABLES: RefCell<RootedTraceableSet> =
         RefCell::new(RootedTraceableSet::new(1234)) ;
 
 ] ;
+
+fn issue3004() {
+    foo!(|_| { (  ) });
+    stringify!(( foo+ ));
+}
+
+// #3331
+pub fn fold_abi<V: Fold + ?Sized>(_visitor: &mut V, _i: Abi) -> Abi {
+    Abi {
+        extern_token: Token ! [ extern ](tokens_helper(_visitor, &_i.extern_token.span)),
+        name: (_i.name).map(|it| _visitor.fold_lit_str(it)),
+    }
+}
